/**
 * @fileoverview BookingContext provides state management for the tattoo booking form.
 * It handles form data, submission state, and related operations.
 */

import React, { createContext, useContext, useReducer } from 'react'

// Create context with null as default value
const BookingContext = createContext(null)

/**
 * Initial state for the booking form
 * @type {Object}
 */
const initialState = {
  formData: {
    name: '',
    email: '',
    phone: '',
    desiredStyle: '',
    size: '',
    placement: '',
    description: '',
    preferredArtist: '',
    preferredDate: '',
    referenceImages: [],
    budget: '',
    hasAllergies: false,
    allergiesDescription: '',
    isFirstTattoo: false,
    agreeToTerms: false,
  },
  isSubmitting: false,
  submitSuccess: false,
  submitError: null,
}

/**
 * Reducer function to handle all booking form state updates
 * @param {Object} state - Current state of the booking form
 * @param {Object} action - Action object with type and payload
 * @returns {Object} New state
 */
function bookingReducer(state, action) {
  switch (action.type) {
    case 'UPDATE_FIELD':
      return {
        ...state,
        formData: {
          ...state.formData,
          [action.field]: action.value,
        },
      }
    case 'UPDATE_MULTIPLE_FIELDS':
      return {
        ...state,
        formData: {
          ...state.formData,
          ...action.fields,
        },
      }
    case 'ADD_REFERENCE_IMAGE':
      return {
        ...state,
        formData: {
          ...state.formData,
          referenceImages: [...state.formData.referenceImages, action.image],
        },
      }
    case 'REMOVE_REFERENCE_IMAGE':
      return {
        ...state,
        formData: {
          ...state.formData,
          referenceImages: state.formData.referenceImages.filter(
            (_, index) => index !== action.index
          ),
        },
      }
    case 'SET_SUBMITTING':
      return {
        ...state,
        isSubmitting: action.isSubmitting,
        submitError: null,
      }
    case 'SET_SUBMIT_SUCCESS':
      return {
        ...state,
        isSubmitting: false,
        submitSuccess: true,
        submitError: null,
      }
    case 'SET_SUBMIT_ERROR':
      return {
        ...state,
        isSubmitting: false,
        submitSuccess: false,
        submitError: action.error,
      }
    case 'RESET_FORM':
      return initialState
    default:
      return state
  }
}

/**
 * Provider component that makes booking context available to any child component
 * @param {Object} props - Component props
 * @param {React.ReactNode} props.children - Child components
 * @returns {JSX.Element} Provider component
 */
export function BookingProvider({ children }) {
  const [state, dispatch] = useReducer(bookingReducer, initialState)

  /**
   * Updates a single field in the form data
   * @param {string} field - Field name to update
   * @param {any} value - New value for the field
   */
  const updateField = (field, value) => {
    dispatch({ type: 'UPDATE_FIELD', field, value })
  }

  /**
   * Updates multiple fields in the form data at once
   * @param {Object} fields - Object containing field names and values
   */
  const updateMultipleFields = (fields) => {
    dispatch({ type: 'UPDATE_MULTIPLE_FIELDS', fields })
  }

  /**
   * Adds a reference image to the form data
   * @param {File|string} image - Image file or URL to add
   */
  const addReferenceImage = (image) => {
    dispatch({ type: 'ADD_REFERENCE_IMAGE', image })
  }

  /**
   * Removes a reference image from the form data
   * @param {number} index - Index of the image to remove
   */
  const removeReferenceImage = (index) => {
    dispatch({ type: 'REMOVE_REFERENCE_IMAGE', index })
  }

  /**
   * Submits the booking form data
   * @param {Object} formData - Form data to submit
   * @returns {Promise<Object>} Result of the submission
   */
  const submitBooking = async (formData) => {
    dispatch({ type: 'SET_SUBMITTING', isSubmitting: true })
    
    try {
      // TODO: Replace with actual API call when backend is ready
      // Simulate API call for now
      await new Promise(resolve => setTimeout(resolve, 2000))
      
      // Log submission for debugging in development
      if (process.env.NODE_ENV === 'development') {
        console.log('Form submitted:', formData)
      }
      
      dispatch({ type: 'SET_SUBMIT_SUCCESS' })
      return { success: true }
    } catch (error) {
      const errorMessage = error?.message || 'An unknown error occurred'
      dispatch({ type: 'SET_SUBMIT_ERROR', error: errorMessage })
      return { success: false, error: errorMessage }
    }
  }

  /**
   * Resets the form to its initial state
   */
  const resetForm = () => {
    dispatch({ type: 'RESET_FORM' })
  }

  const value = {
    ...state,
    updateField,
    updateMultipleFields,
    addReferenceImage,
    removeReferenceImage,
    submitBooking,
    resetForm,
  }

  return (
    <BookingContext.Provider value={value}>
      {children}
    </BookingContext.Provider>
  )
}

/**
 * Custom hook to use the booking context
 * @returns {Object} Booking context value
 * @throws {Error} If used outside of BookingProvider
 */
export function useBooking() {
  const context = useContext(BookingContext)
  if (!context) {
    throw new Error('useBooking must be used within a BookingProvider')
  }
  return context
}

export default BookingContext